(function(){
	var RichEditor = {}
    RichEditor.setup = function(){
		console.log("setup -- ");
        	RichEditor.editor = document.getElementById("editor")
        RichEditor.titleEditor = document.getElementById("title-input")
        RichEditor.touchLocation = null
        $('#editor').on('paste', RichEditor.onPaste)
        $('#editor').on('keydown', RichEditor.onKeyDown)
        $('#editor').on('keypress', RichEditor.onKeyPress)
        $('#editor').on('keyup', RichEditor.onKeyUp)
        $('#editor').on('input',RichEditor.onInput)
        $('#editor').on('blur',RichEditor.onBlur)
        $('#editor').on('focus',RichEditor.onFocus)
        $('#editor').on("DOMNodeInserted",RichEditor.onDomInserted)
        $('#title-input').on('input',RichEditor.titleInput)
        $('#title-input').on('blur',RichEditor.titleBlur)
		$("#video-container").click(function(){
			window.dj.post("mediaItemClicked",{})
		})
		$("#audio-container").click(function(){
			window.dj.post("mediaItemClicked",{})
		})

		if(dj.os.isIOS){
			$("#editor").on("touchstart","figure img",RichEditor.onFigureTouchStart)
			$("#editor").on("touchend","figure img",RichEditor.onFigureTouchEnd)
		}else{
			$("#editor").on("click","figure img",RichEditor.onFigureClick)
		}
	}
	RichEditor.onDomInserted = function(e) {
		$(e.target).removeAttr("style")
    }
    RichEditor.findInsertNode = function(){
        var selection = window.getSelection()
        var insertNode = selection.anchorNode
        while(true){
            if(insertNode.id == "editor" || insertNode.parentNode.id == "editor"){
                break
            }else{
                insertNode = insertNode.parentNode
            }
        }
        return insertNode
    }

    RichEditor.insertParagraph = function(paragraph){
        var insertNode = RichEditor.findInsertNode()
        var innerDiv = $(paragraph)
        if (insertNode.id != 'editor'){
            $(insertNode).after(innerDiv)
            if((insertNode.innerText.trim().length == 0) && (insertNode.tagName == 'P')){
                $(insertNode).remove()
            }
        }else{
            $(insertNode).html(innerDiv)
        }
        RichEditor.moveCarretEnd(innerDiv)
    }

    RichEditor.insertBrAtCurrentPlace = function(){
        var selection = window.getSelection()
        var range = selection.getRangeAt(0)
        var br = document.createElement('br')
        range.deleteContents()
        range.insertNode(br)
        range.setStartAfter(br)
        range.setEndAfter(br)
        range.collapse(false)
        selection.removeAllRanges()
        selection.addRange(range)
    }

    RichEditor.getCaretCharacterOffsetWithin = function(element) {
        var caretOffset = 0;
        var doc = element.ownerDocument || element.document;
        var win = doc.defaultView || doc.parentWindow;
        var sel;
        if (typeof win.getSelection != "undefined") {
            sel = win.getSelection();
            if (sel.rangeCount > 0) {
                var range = win.getSelection().getRangeAt(0);
                var preCaretRange = range.cloneRange();
                preCaretRange.selectNodeContents(element);
                preCaretRange.setEnd(range.endContainer, range.endOffset);
                caretOffset = preCaretRange.toString().length;
            }
        } else if ( (sel = doc.selection) && sel.type != "Control") {
            var textRange = sel.createRange();
            var preCaretTextRange = doc.body.createTextRange();
            preCaretTextRange.moveToElementText(element);
            preCaretTextRange.setEndPoint("EndToEnd", textRange);
            caretOffset = preCaretTextRange.text.length;
        }
        return caretOffset;
    }

    // android
    RichEditor.moveCarretEnd = function(element){
        var selection = window.getSelection()
        var lastChild = element[element.length - 1].lastChild
        selection.collapse(lastChild, lastChild.length /*+ 1*/)

    }
    RichEditor.isCarretAtEnd = function(){
        var element = RichEditor.findInsertNode()
        var offset = RichEditor.getCaretCharacterOffsetWithin(element)
        var length = $(element).text().length
        return offset == length
    }
    RichEditor.isCarretAtBegin = function(){
        var element = RichEditor.findInsertNode()
        var offset = RichEditor.getCaretCharacterOffsetWithin(element)
        return offset == 0 && window.getSelection().getRangeAt(0).startOffset == 0
    }
    //编辑器监听事件
	RichEditor.onInput = function(event){
	    window.dj.post("editorInput",{})
		RichEditor.updatePlaceHolder()
	}
	RichEditor.onBlur = function(event){
        window.dj.post("editorBlur",{})
    }
    RichEditor.onFocus = function(event){
        window.dj.post("editorFocus",{})
    }
	RichEditor.titleInput = function(event){
	    window.dj.post("titleInput",{})
	}
	RichEditor.titleBlur = function(event){
        window.dj.post("titleBlur",{})
    }
	RichEditor.onKeyPress = function(event) {
	    if (event.which == '13' && RichEditor.isCarretAtEnd()) {
		    event.preventDefault()
	        RichEditor.insertParagraph("<p><br/></p>")
	        return
	    }
	}

	RichEditor.onKeyDown = function(event){
        var currentNode = RichEditor.findInsertNode()
        var tagName = currentNode.tagName
        if (event.which == '8'){
            if(tagName == "FIGURE"){
                event.preventDefault()
                if (RichEditor.isCarretAtBegin()){
                    var pre = RichEditor.findInsertNode().previousElementSibling
                    if(pre == undefined){
                        return
                    }
                    if(pre.tagName == "FIGURE"){
                        pre.remove()
                    }else if (pre.tagName == "P"){
                        if(pre.innerText.trim().length == 0){
                            pre.remove()
                        }else{
                            RichEditor.moveCarretEnd($(pre))
                        }
                    }
                }else{
                    currentNode.remove()
                }
            }else if(tagName == 'BLOCKQUOTE'){
                var pre = RichEditor.findInsertNode().previousElementSibling
                if(pre !=undefined && RichEditor.isCarretAtBegin()){
                    if ( pre.tagName == 'BLOCKQUOTE'){
                        event.preventDefault()
                        RichEditor.moveCarretEnd($(pre))
                        $(pre).append($(currentNode).html())
                        currentNode.remove()
                    }
                }
            } else{
                var pre = RichEditor.findInsertNode().previousElementSibling
                if (pre !=undefined && pre.tagName == 'FIGURE' && RichEditor.isCarretAtBegin()) {
                    event.preventDefault()
                    pre.remove()
                }
            }
        }else if (event.which == '13'){
            if (tagName == 'FIGURE'){
                RichEditor.insertParagraph("<p><br/></p>")
            }else if (tagName == 'BLOCKQUOTE'){
                if (RichEditor.isCarretAtBegin()){
                    event.preventDefault()
                    $(currentNode).before("<p><br></p>")
                }else{
                    if (!RichEditor.isCarretAtEnd()){
                        event.preventDefault()
                        RichEditor.insertBrAtCurrentPlace()
                    }
                }
            }
        }else if ( (event.which >= 48 && event.which <= 57)
                || (event.which >= 65 && event.which <= 90)
                || (event.which >= 96 && event.which <= 111)){
            if (tagName == 'FIGURE'){
                RichEditor.insertParagraph("<p><br/></p>")
            }
        }
    }
   RichEditor.onKeyUp = function(event) {
   	    if ($('#editor').html().length == 0) {
   	        RichEditor.insertParagraph("<p><br/></p>")
   	    }
   	}
   	RichEditor.formatBlock = function(type){
   		var currentNode = RichEditor.findInsertNode()
   		var range = document.createRange()
   		range.selectNodeContents(currentNode)
   		var selection = document.getSelection()
   		selection.removeAllRanges()
   		selection.addRange(range)
   		document.execCommand("formatBlock","false",type)
   	}
    RichEditor.toogleHeading = function(){
        RichEditor.hidePlaceholder = true
        var tagName = RichEditor.findInsertNode().tagName
        if (tagName == "P") {
            RichEditor.formatBlock("H4")
        }else if (tagName == "H4"){
            RichEditor.formatBlock("BLOCKQUOTE")
        }else if (tagName == "BLOCKQUOTE"){
            RichEditor.formatBlock("P")
        }
        RichEditor.updatePlaceHolder()
    }
	RichEditor.storeCaret = function(){
        try{
            var selection = window.getSelection();
            var focusRange = selection.getRangeAt(0);
            focusRange.setEnd(focusRange.startContainer, focusRange.startOffset);
            RichEditor.focusRange = focusRange
        }catch(e){}
    }
    RichEditor.restoreCaret = function(){
        try{
            var selection = window.getSelection();
            selection.removeAllRanges();
            selection.addRange(RichEditor.focusRange);
        }catch(e){}
    }

	RichEditor.focus = function(){
		if(RichEditor.titleEditor != null) RichEditor.titleEditor.focus()
	}

	RichEditor.blur = function(){
		RichEditor.storeCaret()
		if(RichEditor.editor != null)  RichEditor.editor.blur()
        if(RichEditor.titleEditor != null) RichEditor.titleEditor.blur()
	}
	RichEditor.getHtml = function(){
	    return dj.nativecallback({callback:"richeditor_html",data:editor.editor.innerHTML})
	}
	RichEditor.getContent = function(){
	    return dj.nativecallback({callback:"richeditor_content",data:editor.editor.textContent.trim()})
	}
	RichEditor.getTitle = function(){
        return dj.nativecallback({callback:"richeditor_title",data:$(editor.titleEditor).val().trim()})
    }
    RichEditor.getImages = function(){
        var images = $("#editor img").map(function(m,n){
            return n.src + ""
         }).toArray()
        return dj.nativecallback({callback:"images",data:images})
    }
    $("article img").click(function(){
        var images = $("article img").map(function(m,n){
            return n.src + ""
        }).toArray()
    })
	RichEditor.updatePlaceHolder = function(){
        var textLen = RichEditor.editor.innerText.trim().length
        var placeholderVisiable = true
        if (RichEditor.hasMedia == true) {
            placeholderVisiable = false
        }
        if(RichEditor.hidePlaceholder == true){
            placeholderVisiable = false
        }
        if (textLen > 0){
            placeholderVisiable = false
        }
        var imageCount = $("#editor img").size()
        if (imageCount > 0){
            placeholderVisiable = false
        }
        if(placeholderVisiable){
            RichEditor.editor.classList.add("placeholder")
        }else{
            RichEditor.editor.classList.remove("placeholder")
        }
    }
	RichEditor.replaceImage = function(source, dest){
        $("#editor figure img").forEach(function(i){
            var src = $(i).attr("src")
            if (src == source){
                $(i).attr("src",dest)
            }
        })
    }

	RichEditor.setPlaceHolder = function(placeholder){
		RichEditor.editor.setAttribute("placeholder", placeholder)
	}

	RichEditor.setContent = function(content){
        if (content != undefined && content.length > 0) {
            RichEditor.editor.innerHTML = content
            RichEditor.updatePlaceHolder()
        }
    }
	RichEditor.insertHtml = function(html){
		document.execCommand('insertHTML', false, html)
		RichEditor.updatePlaceHolder()
	}
	RichEditor.insertImage = function(url){
        var html = '<figure><img src="'+ url +'" alt="" /></figure><p><br></p>';
        RichEditor.restoreCaret()
        RichEditor.insertParagraph(html)
        RichEditor.updatePlaceHolder()
    }
    RichEditor.insertHr = function(){
        var html = '<hr><p><br></p>';
        RichEditor.insertParagraph(html)
    }
	RichEditor.setHasMedia = function(type){
        if(type == "audio"){
            RichEditor.hasMedia = true
            $("#audio-container").removeClass("no-media")
            $("#video-container").addClass("no-media")
        }else if(type == "video"){
            RichEditor.hasMedia = true
            $("#audio-container").addClass("no-media")
            $("#video-container").removeClass("no-media")
        }else{
            RichEditor.hasMedia = false
            $("#audio-container").addClass("no-media")
            $("#video-container").addClass("no-media")
        }
        RichEditor.updatePlaceHolder()
    }
    RichEditor.onVideoClicked = function(event){
        window.dj.post("mediaItemClicked",{})
    }
    RichEditor.onFigureTouchStart = function(event){
        event.stopPropagation();
        RichEditor.touchLocation = {
            x:event.pageX,
            y:event.pageY
        }
    }
    RichEditor.onFigureTouchEnd = function(event){
        event.preventDefault();
        var loc = {
            x:event.pageX,
            y:event.pageY
        }
        var distance = (RichEditor.touchLocation.x - loc.x) * (RichEditor.touchLocation.x - loc.x)
            + (RichEditor.touchLocation.y - loc.y) * (RichEditor.touchLocation.y - loc.y)
        if (distance < 64) {
            var src = $(this).attr("src")
            dj.nativecallback({callback:"imageClicked",data:src}) // android
        }
    }
    RichEditor.onFigureClick = function(){
        var src = $(this).attr("src")
        dj.nativecallback({callback:"imageClicked",data:src}) // android
    }
    RichEditor.onPaste = function(e){
        if(dj.os.isAndroid){
            window.dj.postWithCallback("onPaste",{},function(para){
                var txt = para.text;
                document.execCommand("insertText", 0, txt);

            });
            e.preventDefault();
            e.stopPropagation();

        }else{
            var txt = e.clipboardData.getData("text/plain");
            document.execCommand("insertText", 0, txt);
            e.preventDefault();
            e.stopPropagation();
        }
    }
    this.editor = RichEditor
}).call(window)
$(document).ready(function(){
	window.editor.setup()
})